/*
 * File: IsBalanced.cpp
 * --------------------
 * This program checks to see whether the bracketing operators
 * (parentheses, square brackets, and curly braces) are correctly
 * matched.
 */

#include <iostream>
#include <string>
#include "console.h"
#include "simpio.h"
#include "stack.h"
using namespace std;

/* Function prototypes */

bool isBalanced(string exp);
char matchingClose(char op);

/* Main program */

int main() {
   while (true) {
      string str = getLine("Enter string: ");
      if (str == "") break;
      if (isBalanced(str)) {
         cout << "Brackets are properly nested" << endl;
      } else {
         cout << "Brackets are incorrect" << endl;
      }
   }
   return 0;
}

/*
 * Function: isBalanced
 * Usage: if (isBalanced(str)) . . .
 * ---------------------------------
 * Returns true if the bracketing operators (parentheses, square
 * brackets, and curly braces) are correctly paired in the input string.
 */

bool isBalanced(string str) {
   Stack<char> stack;
   for (int i = 0; i < str.length(); i++) {
      char ch = str[i];
      switch (ch) {
         case '{': case '(': case '[':
            stack.push(ch);
            break;
         case '}': case ')': case ']':
            if (stack.isEmpty()) return false;
            if (ch != matchingClose(stack.pop())) return false;
            break;
      }
   }
   return stack.isEmpty();
}

/*
 * Function: matchingClose
 * Usage: char ch = matchingClose(op);
 * -----------------------------------
 * Returns the matching close operator for the bracketing operator op.
 * If op is not a bracketing operator, this function returns it unchanged.
 */

char matchingClose(char op) {
   switch (op) {
     case '{': return '}';
     case '(': return ')';
     case '[': return ']';
     default: return op;
   }
}
